@@ -13,6 +13,7 @@ from neurom import load_morphology
 from neurom.utils import NeuromJSON
 
 from neuror.cut_plane.detection import CutPlane
+from neuror.exceptions import NeuroRError
 from neuror.unravel import DEFAULT_WINDOW_HALF_LENGTH
 
 logging.basicConfig()
@@ -74,7 +75,7 @@ def file(input_file, output_file, error_summary_file, marker_file):
     from neuror.sanitize import annotate_neurolucida
 
     if Path(input_file).suffix not in ['.asc', '.ASC']:
-        raise Exception('Only .asc/.ASC files are allowed, please convert with morph-tool.')
+        raise NeuroRError('Only .asc/.ASC files are allowed, please convert with morph-tool.')
 
     annotations, summary, markers = annotate_neurolucida(input_file)
     shutil.copy(input_file, output_file)
@@ -170,7 +171,7 @@ def file(input_file, output_file, mapping_file, window_half_length):
     neuron.write(output_file)
     if mapping_file is not None:
         if not mapping_file.lower().endswith('csv'):
-            raise Exception('the mapping file must end with .csv')
+            raise NeuroRError('the mapping file must end with .csv')
         mapping.to_csv(mapping_file)
 
 
@@ -280,7 +281,7 @@ def _export_cut_plane(filename, output, width, display, searched_axes, fix_posit
     It returns the cut plane and the positions of all cut terminations.
 '''
     if os.path.isdir(filename):
-        raise Exception(f'filename ({filename}) should not be a directory')
+        raise NeuroRError(f'filename ({filename}) should not be a directory')
 
     result = CutPlane.find(filename,
                            width,
@@ -303,7 +304,7 @@ def _export_cut_plane(filename, output, width, display, searched_axes, fix_posit
 @compute.command(short_help='Compute a cut plane for morphology FILENAME')
 @click.argument('filename', type=str, required=True)
 @click.option('-o', '--output',
-              help=('Output name for the JSON file (default=STDOUT)'))
+              help='Output name for the JSON file (default=STDOUT)')
 @click.option('-w', '--width', type=float, default=3,
               help='The bin width (in um) of the 1D distributions')
 @click.option('-d', '--display', is_flag=True, default=False,
