@@ -15,6 +15,7 @@ from neurom.morphmath import interval_lengths
 from scipy.spatial import cKDTree
 
 from neuror.cut_plane import CutPlane
+from neuror.exceptions import NeuroRError
 from neuror.utils import RepairJSON
 
 L = logging.getLogger('neuror')
@@ -155,8 +156,8 @@ def unravel_plane(plane, mapping):
     distances, indices = t.query(leaves)
     not_matching_leaves = np.where(distances > 1e-5)[0]
     if not_matching_leaves.size:
-        raise Exception('Cannot find the following leaves in the mapping:\n' +
-                        str(leaves[not_matching_leaves]))
+        raise NeuroRError('Cannot find the following leaves in the mapping:\n' +
+                          str(leaves[not_matching_leaves]))
     plane.cut_leaves_coordinates = mapping.iloc[indices][['x1', 'y1', 'z1']].values
     return plane
 
@@ -168,7 +169,7 @@ def unravel_all(raw_dir, unravelled_dir,
     '''Repair all morphologies in input folder
     '''
     if not os.path.exists(raw_planes_dir):
-        raise Exception(f'{raw_planes_dir} does not exist')
+        raise NeuroRError(f'{raw_planes_dir} does not exist')
 
     if not os.path.exists(unravelled_planes_dir):
         os.mkdir(unravelled_planes_dir)
